#!/usr/bin/env python3
import argparse
import zmsai.base as base
from zmsai.LDA import heuristics
from zmsai.readData import read_txt
import pickle as pkl

import os

import warnings

warnings.filterwarnings("ignore")


def main():
    args = argparse.ArgumentParser()
    args.add_argument("task", nargs="?", default="run", help=base.helpTask)
    args.add_argument("--path", "-p", nargs="?", default=base.path, help=base.helpPath)
    args.add_argument(
        "--topics", "-t", nargs="?", default=base.numberTopics, help=base.helpTopics
    )
    args.add_argument(
        "--nwords", "-w", nargs="?", default=base.nWords, help=base.helpnWords
    )
    args.add_argument(
        "--distro", "-d", nargs="?", default=base.defaultDistro, help=base.helpDistro
    )

    pargs = args.parse_args()
    ndoc, docs = countFiles(pargs.path)
    if pargs.task == "run":
        print(base.run)
        if os.path.exists("meta.zms"):
            os.remove("meta.zms")
        distributions = heuristics(path=pargs.path, numberTopics=int(pargs.topics))
        distributions.save()
        pass
    elif pargs.task == "test":
        print(base.run)
        if os.path.exists("meta.zms"):
            os.remove("meta.zms")
        distributions = heuristics(path=pargs.path, numberTopics=int(pargs.topics))
        distributions.get_doc_topic_distrib()
        distributions.get_topic_word_distrib(int(pargs.nwords))
        distributions.get_doc_word_distrib(int(pargs.nwords))
        distributions.get_vocabulary(int(pargs.nwords))
        pass

    elif pargs.task == "display":
        if os.path.exists("meta.zms"):
            pass
        else:
            choice = input(base.choice)
            if choice == "y":
                path = input("Provide path.")
                print(base.run)
                distributions = heuristics(path=path, numberTopics=int(pargs.topics))
                distributions.save()
            else:
                print("Taking it as a no.")
                return
        pickling_on = open("meta.zms", "rb")
        distributions = pkl.load(pickling_on)

        if pargs.distro == "dt":
            distributions.get_doc_topic_distrib()
        elif pargs.distro == "tw":
            distributions.get_topic_word_distrib(int(pargs.nwords))
        elif pargs.distro == "dw":
            distributions.get_doc_word_distrib(int(pargs.nwords))
        elif pargs.distro == "voc":
            distributions.get_vocabulary(int(pargs.nwords))
        elif pargs.distro == "all":
            distributions.get_doc_topic_distrib()
            distributions.get_topic_word_distrib(int(pargs.nwords))
            distributions.get_doc_word_distrib(int(pargs.nwords))
            distributions.get_vocabulary(int(pargs.nwords))

        pass

    elif pargs.task == "delete":
        pickling_on = open("meta.zms", "rb")
        from sys import getsizeof as size

        obj = pkl.load(pickling_on)
        os.remove("meta.zms")
        print(base.delete, size(obj) / 1024, "kb freed")
        pass

    elif pargs.task == "man":
        USAGE = open("MAN.txt", "r")
        print(USAGE.read())
        pass

    else:
        print("[Invalid argument]")
        pass


def countFiles(DIR):
    from os import listdir

    lsdir = listdir(DIR)
    count = len(lsdir)
    return count, lsdir


if __name__ == "__main__":
    main()



from zmsai.readData import read_txt
from zmsai.utils import print_topics
from zmsai.utils import print_list
from zmsai.utils import padding
from zmsai.utils import countFiles

import warnings

warnings.simplefilter("ignore", DeprecationWarning)


class heuristics:
    def __init__(self, path, numberTopics):
        data = read_txt(path)
        self.ndocs, self.docs = countFiles(path)
        self.vector, self.vectorizer = self.get_feature_space(data)
        self.transform, self.lda = self.LDA_transform(numberTopics)

    def get_topic_word_distrib(self, nWords):
        topic_word = self.lda.components_
        print_topics(self.lda, self.vectorizer, nWords)
        return topic_word

    def get_doc_word_distrib(self, nWords):
        print("=====================")
        print("DOC-WORD DISTRIBUTION")
        print("=====================")
        message = "Words\t\tCount"
        import numpy as np

        words = self.vectorizer.get_feature_names()
        total_counts = np.zeros(len(words))
        for i, t in enumerate(self.vector):
            padding(len("* " + self.docs[i]), "-")
            print("*", self.docs[i])
            padding(len("* " + self.docs[i]), "-")
            total_counts = t.toarray()[0]
            count_dict = zip(words, total_counts)
            count_dict = sorted(count_dict, key=lambda x: x[1], reverse=True)[0:nWords]
            print_list(count_dict, message)
            print("\n")
        return count_dict

    def get_doc_topic_distrib(self):
        print("======================")
        print("DOC-TOPIC DISTRIBUTION")
        print("======================")
        jstr = ""
        nPad = 15
        ls = [" "] * (nPad - len("DOCS"))
        pad = jstr.join(ls)

        print("\nDocs", pad, "Topics")
        for n in range(self.transform.shape[0]):
            topic_most_pr = self.transform[n].argmax()

            ls = [" "] * (nPad - len(self.docs[n]))
            pad = jstr.join(ls)

            print(self.docs[n], pad, topic_most_pr + 1)
        print("\n")

    def get_vocabulary(self, nWords):
        from numpy import zeros as npz

        print("===========================")
        print("CORPUS VOCABULARY DISTRIBUTION")
        print("===========================")
        message = "Words\t\tCount"

        words = self.vectorizer.get_feature_names()
        total_counts = npz(len(words))
        for t in self.vector:
            total_counts += t.toarray()[0]

        count_dict = zip(words, total_counts)
        count_dict = sorted(count_dict, key=lambda x: x[1], reverse=True)[0:nWords]
        print_list(count_dict, message)
        print("\n")
        return self.vectorizer.vocabulary_

    def get_feature_space(self, data):
        from sklearn.feature_extraction.text import CountVectorizer

        vectorizer = CountVectorizer(stop_words="english")
        vector = vectorizer.fit_transform(data)
        return vector, vectorizer

    def LDA_transform(self, numberTopics):
        from sklearn.decomposition import LatentDirichletAllocation as LDA

        lda = LDA(n_components=numberTopics)
        transform = lda.fit_transform(self.vector)
        return transform, lda

    def save(self):
        import pickle as pkl
        from sys import getsizeof as size

        pickling_on = open("meta.zms", "wb")
        pkl.dump(self, pickling_on)
        pickling_on.close()
        print("[Data stored]", size(self) / 1024, "kb")
        pass



# -*- coding: utf-8 -*-
import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="zmsai",
    version="0.1.2",
    author="evi1haxor",
    author_email="architdwivedi.off@gmail.com",
    description="Ai solution for Luhmann's Zettelkasten",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/evi1haxor/zettelkasten-ai/",
    packages=["zmsai"],
    # packages=setuptools.find_packages(),
    license="GPLv3",
    classifiers=[
        "Programming Language :: Python :: 3.8",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Operating System :: OS Independent",
    ],
    entry_points={"console_scripts": ["zmsai=zmsai.__main__:main",]},
)



from os import walk, listdir
from zmsai.utils import series_to_list as stl


def read_txt(path="./custom"):
    files = []
    data = dict()
    for (dirpath, dirnames, filenames) in walk(path):
        files.extend([str(dirpath) + "/" + i for i in filenames])
    fstream = list(map(open, files))
    txt = [i.read() for i in fstream]
    return txt


def read_csv(path="./custom/data.csv", sep=","):
    import pandas as pd

    files = pd.read_csv(path, delimiter=sep)
    txt = stl(files["text"])
    return txt
